/**
 * Copyright 2010 Västra Götalandsregionen
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of version 2.1 of the GNU Lesser General Public
 *   License as published by the Free Software Foundation.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 *   Boston, MA 02111-1307  USA
 */

package se.vgregion.ifeed.types;

import net.sf.cglib.beans.BeanMap;
import se.vgregion.dao.domain.patterns.entity.AbstractEntity;

import javax.persistence.*;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

@Entity
@Table(name = "vgr_fields_inf")
public class FieldsInf extends AbstractEntity<Long> implements Serializable, Comparable<FieldsInf> {

    private static final long serialVersionUID = 1L;

    public FieldsInf() {
        // For the jpa.
    }

    @Id
    @GeneratedValue
    private Long id;

    @Version
    private Long version;

    private String creatorId;

    @Column(length = 100000)
    private String text;

    @Override
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public Long getVersion() {
        return version;
    }

    public void setVersion(Long version) {
        this.version = version;
    }

    public String getCreatorId() {
        return creatorId;
    }

    public void setCreatorId(String creatorId) {
        this.creatorId = creatorId;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    public List<FieldInf> getFieldInfs() {
        List<FieldInf> result = new ArrayList<FieldInf>();

        String t = getText();
        String[] rows = t.split(Pattern.quote("\n"));

        Map<Integer, String> fieldPosition = new HashMap<Integer, String>();
        String[] first = rows[0].split(Pattern.quote("|"));

        for (int i = 0; i < first.length; i++) {
            fieldPosition.put(i, first[i].trim());
        }

        List<FieldInf> nestedResult = result;

        for (int i = 1; i < rows.length; i++) {
            String row = rows[i];
            String[] cells = row.split(Pattern.quote("|"));
            FieldInf fi = new FieldInf();

            BeanMap bm = BeanMap.create(fi);
            for (int c = 0, j = fieldPosition.size(); c < j; c++) {
                if (c >= cells.length) {
                    continue;
                }
                String name = fieldPosition.get(c);
                Class<?> type = bm.getPropertyType(name);

                if (type.equals(Boolean.TYPE)) {
                    bm.put(name, "yes".equalsIgnoreCase(cells[c].trim()));
                } else if (type.equals(List.class) && name.equals("children")) {
                    ((List) bm.get(name)).add(cells[c].trim());
                } else {
                    bm.put(name, cells[c].trim());
                }
            }

            if (fi.isLabel()) {
                nestedResult = fi.getChildren();
                result.add(fi);
            } else {
                fi.setId(fi.getId().toLowerCase());
                nestedResult.add(fi);
            }
        }

        return result;
    }

    @Override
    public int compareTo(FieldsInf o) {
        return (int) (id - o.id);
    }

}
